#!/usr/bin/env python
from webob.response import Response

def make_middleware(app):
    from repoze.profile.profiler import AccumulatingProfileMiddleware
    return AccumulatingProfileMiddleware(
        app,
        log_filename='/tmp/profile.log',
        discard_first_request=True,
        flush_at_shutdown=True,
        path='/__profile__')

def simple_app(environ, start_response):
    resp = Response('Hello world!')
    return resp(environ, start_response)

if __name__ == '__main__':
    import sys
    import os
    import signal
    if sys.argv[1:]:
        arg = sys.argv[1]
    else:
        arg = None
    if arg in ['open', 'run']:
        import subprocess
        import webbrowser
        import time
        os.environ['SHOW_OUTPUT'] = '0'
        proc = subprocess.Popen([sys.executable, __file__])
        time.sleep(1)
        subprocess.call(['ab', '-n', '1000', 'http://localhost:8080/'])
        if arg == 'open':
            webbrowser.open('http://localhost:8080/__profile__')
        print('Hit ^C to end')
        try:
            while 1:
                raw_input()
        finally:
            os.kill(proc.pid, signal.SIGKILL)
    else:
        from paste.httpserver import serve
        if os.environ.get('SHOW_OUTPUT') != '0':
            print('Note you can also use:)')
            print('  %s %s open' % (sys.executable, __file__))
            print('to run ab and open a browser (or "run" to just run ab)')
            print('Now do:')
            print('ab -n 1000 http://localhost:8080/')
            print('wget -O - http://localhost:8080/__profile__')
        serve(make_middleware(simple_app))



import pytest

from webob.etag import IfRange, ETagMatcher
from webob import Response

def test_if_range_None():
    ir = IfRange.parse(None)
    assert str(ir) == ''
    assert not ir
    assert Response() in ir
    assert Response(etag='foo') in ir
    assert Response(etag='foo GMT') in ir

def test_if_range_match_date():
    date = 'Fri, 09 Nov 2001 01:08:47 GMT'
    ir = IfRange.parse(date)
    assert str(ir) == date
    assert Response() not in ir
    assert Response(etag='etag') not in ir
    assert Response(etag=date) not in ir
    assert Response(last_modified='Fri, 09 Nov 2001 01:00:00 GMT') in ir
    assert Response(last_modified='Fri, 10 Nov 2001 01:00:00 GMT') not in ir

def test_if_range_match_etag():
    ir = IfRange.parse('ETAG')
    assert str(ir) == '"ETAG"'
    assert Response() not in ir
    assert Response(etag='other') not in ir
    assert Response(etag='ETAG') in ir
    assert Response(etag='W/"ETAG"') not in ir

def test_if_range_match_etag_weak():
    ir = IfRange.parse('W/"ETAG"')
    assert str(ir) == ''
    assert Response(etag='ETAG') not in ir
    assert Response(etag='W/"ETAG"') not in ir

def test_if_range_repr():
    assert repr(IfRange.parse(None)) == 'IfRange(<ETag *>)'
    assert str(IfRange.parse(None)) == ''

def test_resp_etag():
    def t(tag, res, raw, strong):
        assert Response(etag=tag).etag == res
        assert Response(etag=tag).headers.get('etag') == raw
        assert Response(etag=tag).etag_strong == strong
    t('foo', 'foo', '"foo"', 'foo')
    t('"foo"', 'foo', '"foo"', 'foo')
    t('a"b', 'a"b', '"a\\"b"', 'a"b')
    t('W/"foo"', 'foo', 'W/"foo"', None)
    t('W/"a\\"b"', 'a"b', 'W/"a\\"b"', None)
    t(('foo', True), 'foo', '"foo"', 'foo')
    t(('foo', False), 'foo', 'W/"foo"', None)
    t(('"foo"', True), '"foo"', r'"\"foo\""', '"foo"')
    t(('W/"foo"', True), 'W/"foo"', r'"W/\"foo\""', 'W/"foo"')
    t(('W/"foo"', False), 'W/"foo"', r'W/"W/\"foo\""', None)

def test_matcher():
    matcher = ETagMatcher(['ETAGS'])
    matcher = ETagMatcher(['ETAGS'])
    assert matcher.etags == ['ETAGS']
    assert "ETAGS" in matcher
    assert "WEAK" not in matcher
    assert "BEER" not in matcher
    assert None not in matcher
    assert repr(matcher) == '<ETag ETAGS>'
    assert str(matcher) == '"ETAGS"'

    matcher2 = ETagMatcher(("ETAG1","ETAG2"))
    assert repr(matcher2) == '<ETag ETAG1 or ETAG2>'



from webob.request import Request, Transcoder
from webob.response import Response
from webob.compat import text_, native_

t1 = b'--BOUNDARY\r\nContent-Disposition: form-data; name="a"\r\n\r\n\xea\xf3...\r\n--BOUNDARY--'
t2 = b'--BOUNDARY\r\nContent-Disposition: form-data; name="a"; filename="file"\r\n\r\n\xea\xf3...\r\n--BOUNDARY--'
t3 = b'--BOUNDARY\r\nContent-Disposition: form-data; name="a"; filename="\xea\xf3..."\r\n\r\nfoo\r\n--BOUNDARY--'

def test_transcode():
    def tapp(env, sr):
        req = Request(env)
        req = req.decode()

        v = req.POST[req.query_string]

        if hasattr(v, 'filename'):
            r = Response(text_('%s\n%r' % (v.filename, v.value)))
        else:
            r = Response(v)
        return r(env, sr)

    text = b'\xea\xf3...'.decode('cp1251')
    def test(post):
        req = Request.blank('/?a', POST=post)
        req.environ['CONTENT_TYPE'] = 'multipart/form-data; charset=windows-1251; boundary=BOUNDARY'
        return req.get_response(tapp)

    r = test(t1)
    assert r.text == text
    r = test(t2)
    assert r.text == 'file\n%r' % text.encode('cp1251')
    r = test(t3)
    assert r.text, "%s\n%r" % (text == b'foo')

def test_transcode_query():
    req = Request.blank('/?%EF%F0%E8=%E2%E5%F2')
    req2 = req.decode('cp1251')
    assert req2.query_string == '%D0%BF%D1%80%D0%B8=%D0%B2%D0%B5%D1%82'

def test_transcode_non_multipart():
    req = Request.blank('/?a', POST='%EF%F0%E8=%E2%E5%F2')
    req._content_type_raw = 'application/x-www-form-urlencoded'
    req2 = req.decode('cp1251')
    assert native_(req2.body) == '%D0%BF%D1%80%D0%B8=%D0%B2%D0%B5%D1%82'

def test_transcode_non_form():
    req = Request.blank('/?a', POST='%EF%F0%E8=%E2%E5%F2')
    req._content_type_raw = 'application/x-foo'
    req2 = req.decode('cp1251')
    assert native_(req2.body) == '%EF%F0%E8=%E2%E5%F2'

def test_transcode_noop():
    req = Request.blank('/')
    assert req.decode() is req

def test_transcode_query_ascii():
    t = Transcoder('ascii')
    assert t.transcode_query('a') == 'a'
